#ifndef ORB2ORB_CONFIG_H
#define ORB2ORB_CONFIG_H

#include "brttutil.h"

#include "orbtag.h"

#define EPOCHTIME_NULL -9999999999.999
#define EPOCHTIME_LAST -8888888888.888
#define EPOCHTIME_STRING_FORMAT (char*) "%D %T %Z" 

class O2oLogMgr;
class O2oStateMgr;
class O2oStatusMgr;
class O2oQueue;
class O2oConnection;

enum O2oUsageMode { current, legacy };

class O2oConfig 
{
	public:
										O2oConfig( void );
										~O2oConfig( void );

		void								configure( int argc, char** argv );

		void								nameThread( std::string name );

		bool								isLegacy( void );
		int								getInt( std::string key );
		double								getDouble( std::string key );
		O2oLogMgr*							omlog( void );
		O2oStateMgr*							omstate( void );
		O2oStatusMgr*							omstatus( void );
		std::string							getMyThreadName( void );
		std::string							getMyThreadLabel( void );
		std::string							getStateFile( void );
		std::string							getTargetName( void );
		std::shared_ptr<O2oQueue>					getO2oQueue( std::string queueName );
		std::shared_ptr<O2oQueue>					getO2oLogQueue();
		std::shared_ptr<O2oQueue>					getO2oStatusQueue();
		std::vector<std::shared_ptr<O2oConnection>>			getConnections( void );

	private: 				
		
		void								usage( void );
		void								addQueue( std::string queueName );
		std::shared_ptr<O2oConnection>					addConnection( Pf* pf );
		void								addOrb( std::string orbname, std::string orbtag = "" );
		void								configureFinishLegacyCmd( int argc, char** argv );
		void								configureFinishCurrentCmd( int argc, char** argv );
		void								configureFinishLegacyPf( Pf* pf );
		void								configureFinishCurrentPf( Pf* pf );
		void								vetConnectionPf( Pf* pf );
		std::string							determineConnectionDirection( Pf* pf );

		O2oUsageMode							m_usageMode;
		O2oLogMgr*							m_omlog;
		O2oStateMgr*							m_omstate;
		O2oStatusMgr*							m_omstatus;
		double								m_startTime;
		double								m_pfstatusReportIntervalSec;
		double								m_internalTimeoutSec;
		int								m_shutdownGracePeriodSec;
		int								m_verbose;
		int								m_longestThreadNameLength;
		std::string							m_pfName;
		std::string							m_targetName;
		std::string							m_stateFile;
		std::string							m_statusQueue;
		std::string							m_logQueue;
		char*								m_legacyOrbnameIn;
		char*								m_legacyOrbnameOut;
		char*								m_legacyMatch;
		char*								m_legacyReject;
		double								m_legacyStarttime;
		double								m_legacyEndtime;

		std::unordered_map<std::string,std::shared_ptr<O2oQueue>>	m_queues;
		std::unordered_map<std::string,std::shared_ptr<O2oConnection>>	m_connections;
		std::unordered_map<std::thread::id,std::string>			m_threadNames;

};

extern O2oConfig* O2oCnf;

#endif
